import requests

class dotdict(dict):
    """Wrapper class that adds dot notation for dict."""
    def __getattr__(self, name):
        if type(self.get(name)) == dict:
            return dotdict(self.get(name))
        return self.get(name)

def create_group(name, path, description, headers, url):
    """Create a group and return its ID or return the group ID if it already exists.
        
    Args:
        name (str): Group name.
        path (str): Group path.
        description (str): Group description.
        headers (dict): Request headers.
        url (str): GitLab API URL.
    
    Returns:
        int: Group ID.
    """
    
    data = {
        "name": name,
        "path": path,
        "description": description
    }
    print(f"{url}/groups/{path}")
    r = requests.get(f"{url}/groups/{path}", headers=headers)
    if r.status_code == 200:
        print(f"Group {path} already exists (id: {r.json()['id']})", )
        return r.json()["id"]

    r = requests.post(f"{url}/groups", data=data, headers=headers)
    if r.status_code == 201:
        print(f"Group {path} has been created (id: {r.json()['id']})")
        return r.json()["id"]
    else:
        raise Exception(r.status_code, r.json())

def create_subgroup(name, path, description, parent_id, headers, url):
    """Create a group and return its ID or return the ID if the group already exists.
            
    Args:
        name (str): Subgroup name.
        path (str): Subgroup path.
        description (str): Subgroup description.
        parent_id (int): Parent group ID.
        headers (dict): Request headers.
        url (str): GitLab API URL.
    
    Returns:
        int: Subgroup ID.
    """
    
    data = {
        "name": name,
        "path": path,
        "description": description,
        "parent_id": parent_id
    }

    r = requests.get(f"{url}/groups/{parent_id}/subgroups", headers=headers)
    if r.status_code == 200:
        for group in r.json():
            if group["path"] == path:
                print(f"Subgroup {path} already exists (id: {group['id']})")
                return group["id"]
    else:    
        raise Exception(r.status_code, r.json()["message"])
    
    r = requests.post(f"{url}/groups", data=data, headers=headers)
    if r.status_code == 201:
        print(f"Subgroup {path} has been created (id: {r.json()['id']})")
        return r.json()["id"]
    else:
        raise Exception(r.status_code, r.json()["message"])

def create_project(namespace_id, name, path, template_url, headers, url):
    """Create a project and return its ID or return the ID if the project already exists.
    
    Args:
        namespace_id (int): Namespace ID.
        name (str): Project name.
        path (str): Project path.
        template_url (str): URL reference to a project that will be used as a template.
        headers (dict): Request headers.
        url (str): GitLab API URL.
    
    Returns:
        int: Project ID.
    """

    data = {
        "name": name,
        "path": path,
        "namespace_id": namespace_id,
        "import_url": template_url,
        "initialize_with_readme": template_url == None
    }

    project_id = get_project_id(namespace_id, path, headers, url)
    if project_id != None:
          print(f"Project {path} already exists (id: {project_id})")
          return project_id
    
    r = requests.post(f"{url}/projects", data=data, headers=headers)
    if r.status_code == 201:
          print(f"Project {path} has been created (id: {r.json()['id']})")
          return r.json()["id"]
    else:
          raise Exception(r.status_code, r.json()["message"])
      
def get_project_id(namespace_id, path, headers, url):
    """Get a project and return its ID or return None if the project could not be found.
    
    Args:
        namespace_id (int): Namespace ID. 
        path (str): Project path.
        headers (dict): Request headers.
        url (str): GitLab API URL.
    
    Returns:
        int: Project ID if the project is found, otherwise None.
    """
    
    r = requests.get(f"{url}/groups/{namespace_id}/projects/?search={path}", headers=headers)
    if len(r.json()) > 0:
        return r.json()[0]["id"]
    return None

def generate_projects(prefixes, suffixes, namespace_id, template_url, headers, url):
    """Generate projects based on list of prefixes suffixes.
    
    Args:
        prefixes (str[]]): List or prefixes.
        suffixes (str[]): List of suffixes.
        namespace_id (int): Namespace ID.
        template_url (str): Template project URL.
        headers (dict): Request headers.
        url (str): GitLab API URL.
    
    Returns:
        None
    """
    
    project_names = [f"{p}{s}" for p in prefixes for s in suffixes]    
    for project_name in project_names:
      project_path = project_name
      create_project(namespace_id, project_name, project_path, template_url, headers, url)

def add_member(project_id, path, user, headers, url):
    """Add a user as a maintainer of a project.
    
    Args:
        namespace_id (int): Namespace ID. 
        path (str): Project path.
        user (in): User ID.
        headers (dict): Request headers.
        url (str): GitLab API URL.
    
    Returns:
        None
    """
    r = requests.get(f"{url}/users/?username={user}", headers=headers)
    if r.json() == []:
        print(f"User {user} not found")
        return
    user_id = r.json()[0]["id"]
    
    data = {
        "user_id": user_id,
        "access_level": 40 # manager
    }
    r = requests.post(f"{url}/projects/{project_id}/members", data=data, headers=headers)
    if r.status_code == 201:
        print(f"Added {user} as maintainer to {path}")
    else:
        print(r.json()["message"]) 

def add_members(path, namespace_id, headers, url, template_url=None, init_projects=False):
    """Add users as a maintainers to project.
    
    The project names are parsed from the webreg file.
    
    Args:
        path (str): Project path.
        namespace_id (int): Namespace ID. 
        headers (dict): Request headers.
        url (str): GitLab API URL.
    
    Returns:
        None
    """
    with open(path, "r") as f:
        for line in f.readlines():
            line = line.strip()
            if line == "":
                continue
            parts = line.split(":")
            user = parts[0]
            path = f"{parts[-2]}{parts[-1]}"
            
            if init_projects:
                name = path
                project_id = create_project(namespace_id, name, path, template_url, headers, url)
            else:
                project_id = get_project_id(namespace_id, path, headers, url)
                if project_id == None:
                    print(f"Project {path} does not exist")
                    continue
            
            add_member(project_id, path, user, headers, url)


